# i18n support
##################################################
## DEPENDENCIES ##

from django.core import template
from contrib.i18n.i18n import get_translation

##################################################
## TAGS ##

class I18N_Node(template.Node):
    def __init__(self, cmd):
        self.cmd = cmd
    #

    def render(self, context):
        # make locales for eval
        loc = {}
        for d in context.dicts:
            loc.update(d)
        #

        i18n = get_translation()

        # check command
        cmd, _ = self.cmd.split('(', 1)
        if getattr(i18n, cmd, None) is None: # not a i18n command
            raise TypeError("'i18n' only i18n commands are allowed (got %r:%r)" % (i18n, cmd))
        #

        try:
            loc['i18n'] = i18n
            msg =  eval('i18n.' + self.cmd, {}, loc)
            return msg % loc
        except Exception, err:
            raise  err.__class__, 'i18n.%s: %s (%r)' % (self.cmd, err, loc)
    #

def do_i18n(parser, token):
    """ call i18n functions from current context

    {% i18n gettext('test') %} will be called as
    context['i18n'].gettext('test')
    """
    import warnings
    warnings.warn('i18n tag is depreciated, use gettext', DeprecationWarning, stacklevel=2)

    args = token.contents.split(' ', 1)
    if len(args) != 2:
        raise template.TemplateSyntaxError("'i18n' statement requires one argument (got %r)" % args)

    return I18N_Node(args[1].strip())
# do_i18n

class GetTextNode(template.Node):
    def __init__(self, var, nodelist_singular, nodelist_plural):
        self.nodelist_singular = nodelist_singular
        self.nodelist_plural = nodelist_plural
        self.var = var
    #

    def render(self, context):
        i18n = get_translation()

        # singular
        bits = []
        recompile = False
        for node in self.nodelist_singular:
            if isinstance(node, template.VariableNode):
                # treat VariableNodes specially
                bits.append('{{ %s }}' % node.var_string)
                recompile = True
            elif isinstance(node, template.Node):
                bits.append(node.render(context))
            else:
                bits.append(node)
        # for
        singular = ''.join(bits)

        if self.nodelist_plural is not None:
            # plural
            bits = []
            for node in self.nodelist_plural:
                if isinstance(node, template.VariableNode):
                    # treat VariableNodes specially
                    bits.append('{{ %s }}' % node.var_string)
                    recompile = True
                elif isinstance(node, template.Node):
                    bits.append(node.render(context))
                else:
                    bits.append(node)
            # for
            plural = ''.join(bits)

            val = template.resolve_variable(self.var, context)
            msg = i18n.ngettext(singular, plural, val)
        else:
            msg = i18n.gettext(singular)

        # vars detected, recompile message
        if recompile:
            return template.Template(msg).render(context)
        else:
            return msg
    # render
# NGetTextNode

def do_gettext(parser, token):
    """
    Get translation for containing message.
    If you need to use only gettext, you can use
    *translate* filter.

    Example singular::

        {% gettext %}
            ...
        {% endgettext %}

    Example plural::

        {% gettext val %}
            ...
        {% plural %}
            ...
        {% endgettext %}
    """

    bits = token.contents.split()

    if len(bits) == 2:
        var = bits[1]
    else:
        var = None
    #

    end_tag = 'end' + bits[0]
    nodelist_singular = parser.parse(('plural', end_tag))
    token = parser.next_token()

    if token.contents == 'plural':
        if var is None:
            raise template.TemplateSyntaxError, "%r takes one argument with plural part" % bits[0]
        #
        nodelist_plural = parser.parse((end_tag,))
        parser.delete_first_token()
    else:
        if var is not None:
            raise template.TemplateSyntaxError, "%r takes no arguments without plural part" % bits[0]
        #
        nodelist_plural = None

    return GetTextNode(var, nodelist_singular, nodelist_plural)
# do_gettext

template.register_tag('i18n', do_i18n)
template.register_tag('gettext', do_gettext)

##################################################
## FILTERS ##

def translate(value, _):
    """ Given a string returns translated message using current translation
        (gettext)
    """
    i18n = get_translation()
    return i18n.gettext(value)
#

template.register_filter('translate', translate, False)